'''
.. module:: var
   :platform: Linux
   :synopsis: The var module contains the VAR class for estimating and doing further work with Vector Autoregressions commonly
              used in applied macroeconometrics. It supports advanced methods such as bootstrapping confidence intervals including
              Killian's boostrap-after-bootstrap small-sample bias correction. Also CPU-intensive methods such as the bootstrap can
              be computed using Parallel Python to exploit multi-core CPUs. Pretty plotting methods are also included which depend
              on matplotlib.

.. moduleauthor:: Eric M. Scheffel <eric.scheffel@nottingham.edu.cn>


'''
import numpy
import scipy
import scipy.stats
# We don't need this anywhere and it should also be replaced by Pandas in the future
#from scikits import timeseries as ts
import matplotlib as mpl
from matplotlib import rc
rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
## for Palatino and other serif fonts use:
#rc('font',**{'family':'serif','serif':['Palatino']})
#rc('text', usetex=True)
from matplotlib import pyplot as plt
from matplotlib import pylab as pyl
import copy
import datetime
import pickle
import pp
import time
import os

# Re-factored imports
from ..dattrans.transmeth import stdX, transX
from .common import genXX, compbetta

# Switch off runtime warnings here
import warnings
warnings.filterwarnings("ignore")


class VAR:

    def __init__(self,data=None,vnames=None,pnames=None,svnames=None,irfs=True,boot=True,plot=True,conf=None,mesg=False):
        print '#####################################################'
        stepo=0
        stepo+=1
        self._mesg = mesg
        if self._mesg: print str(stepo)+') Initialising VAR'
        self.confdic = conf
        params = {'axes.labelsize':conf['graph_options']['labels']['label_fs'],
                  'text.fontsize': conf['graph_options']['labels']['text_fs'],
                  'legend.fontsize': conf['graph_options']['labels']['legend_fs'],
                  'xtick.labelsize': conf['graph_options']['labels']['xticks_fs'],
                  'ytick.labelsize': conf['graph_options']['labels']['yticks_fs'],
                  'text.usetex': conf['graph_options']['labels']['use_tex'],
                  'lines.linewidth': conf['graph_options']['lines']['width']}
        pyl.rcParams.update(params)        
        self.vnames = vnames
        self.pnames = pnames
        self.svnames = svnames
        self.const_term = conf['const_term']
        self.nlags = conf['nlags']
        self.freq = conf['freq']
        self.transperc_dic = conf['transperc_dic']
        self.tcode_dic = copy.deepcopy(conf['tcode'])
        self.data = data
        self.ncols = data.shape[1]
        self.nrows = data.shape[0]
        # Create irf_dic for graphs
        self.irf_graphs_dic = {}
        for sname in vnames:
            self.irf_graphs_dic[sname] = {}
            for rname in vnames:
                self.irf_graphs_dic[sname][rname] = None
        ############################################################
        if self._mesg:
            print "This VAR model's name is: "+self.confdic['modname']
            print 'The dataset used is: '+self.confdic['datafile']
            print 'The chosen autoregressive order is: '+str(self.nlags)
            print 'The total number of variables in the system is: '+str(self.ncols)
            print 'The total number of observations before estimation is: '+str(self.nrows)
            if conf['const_term'] == None:
                print 'The model was estimated using neither a constant nor a time-trend'
            elif conf['const_term'] == 'cc':
                print 'The model was estimated using only a constant'
            elif conf['const_term'] == 'tt':
                print 'The model was estimated using only a time-trend'
            elif conf['const_term'] == 'ct':
                print 'The model was estimated using both a constant and a time-trend'
            elif conf['const_term'] == 'ctt':
                print 'The model was estimated using both a constant, a linear, and a quadratic time trend'
            if conf['translog']:
                print "The model's impulse responses are in % changes for vars in logs"
            elif not conf['translog']:
                print "The model's impulse responses are in abs. log dev. for vars in logs"
            if conf['use_svnames']:
                print "The model's impulse responses are based on a customized shock matrix"
            elif not conf['use_svnames']:
                print "The model's impulse responses are based on the identity shock matrix"
            print 'The frequency of the data is: '+str(self.freq)
            print '#####################################################'
        stepo+=1
        if self._mesg: print str(stepo)+') Standardising data'
        # Transform raw data according to Stock and Watson's trans code encoding system
        if self._mesg: print 'Now TRANSFORMING data according to Stock and Watson transformation encoding...'
        self = transX(self,data=self.data)
        if self.confdic['trans_data']:
            # Standardize the data if wanted, also store means and standard errors
            if self._mesg: print 'Now STANDARDIZING data ahead of estimation procedure...'
            self = stdX(self,data=self.data,standard=True)
            self.data = copy.deepcopy(self.tdata)
        else:
            self = stdX(self,data=self.data,standard=False)
        if self._mesg: print '#####################################################'
        stepo+=1
        if self._mesg: print str(stepo)+') Estimate coefficient matrix of the VAR'
        self = genXX(self)
        compbetta(self)
        if self.confdic['const_term'] != None:
            self.compbettax()
        if self._mesg: print '#####################################################'
        stepo+=1
        if self._mesg: print str(stepo)+') Creating companion matrices'
        self.mkCompMat()
        self.mkfitres()
        self.mksigma()
        if irfs:
            if self._mesg: print 'Now generating impulse responses...'
            self.mkphis()       
        if irfs and boot:
            addstr = ''
            if self.confdic['multicpu']: addstr = '(parallel execution)'
            else: addstr = '(serial execution)'
            if self._mesg: print 'Now generating bootstrap confidence intervals'+addstr+'...'
            t0 = time.time()            
            if not self.confdic['multicpu']: self.mkboot()
            elif self.confdic['multicpu']: self.mkboot_pp()
            if self._mesg: print round(time.time() - t0,2), "secs used to execute..."
        if self.confdic['do_vdc']:
            vdcp = self.confdic['vdcp']
            self.genFEVD(self.data,self.bettasm,vdcp) 
        if plot:
            if self._mesg: print 'Now plotting and saving graphs of actual, fitted and residual data for all series...'
            self.plot_vals()
        if irfs:
            if self._mesg: print 'Now plotting and saving graphs of impulse responses...'
            self.plot_irfs()
        if self._mesg: print '*All done !*'
   

    # Computes coefficient matrix but with a column of ones in the Y matrix.
    # This is useful for when there is a constant in the coefficient matrix
    def compbettax(self,matd=None,nmatd=None,smbetta=True,const='standard',func=False):
        # Get all the needed variables from instance
        if matd == None: matd = self.data
        if nmatd == None: nmatd = self.nmatd
        if const == 'standard': const_term = self.confdic['const_term']
        cols = matd.shape[1]
        if const_term == None:
            nlags = int(nmatd.shape[1]/cols)
        elif const_term == 'cc' or const_term == 'tt':
            nlags = int((nmatd.shape[1]-1)/cols)
            # Add column of ones or time index to the ymat
            if const_term == 'cc': matd = numpy.hstack((numpy.ones((matd.shape[0],1)),matd))
            elif const_term == 'tt': matd = numpy.hstack((numpy.arange(0,matd.shape[0],1.0).reshape(len(numpy.arange(matd.shape[0])),1),matd))
        elif const_term == 'ct':
            # Add column of ones and column of time index to ymat, so two extra columns
            nlags = int((nmatd.shape[1]-2)/cols)
            const = numpy.ones((matd.shape[0],1))
            timet = numpy.arange(0,matd.shape[0],1.0).reshape(len(numpy.arange(matd.shape[0])),1)
            matd = numpy.hstack((const,timet,matd))
        elif const_term == 'ctt':
            # Add column of ones and column of time index to ymat, so two extra columns
            nlags = int((nmatd.shape[1]-3)/cols)
            const = numpy.ones((matd.shape[0],1))
            timet = numpy.arange(0,matd.shape[0],1.0).reshape(len(numpy.arange(matd.shape[0])),1)
            timet2 = timet**2
            matd = numpy.hstack((const,timet,timet2,matd))
        fmatd = nmatd
        XX = numpy.dot(fmatd.T,fmatd)
        XXi = numpy.linalg.inv(XX)
        if const_term == None:
            betta = numpy.zeros((cols,nlags*cols))
        elif const_term == 'tt' or const_term == 'cc':
            betta = numpy.zeros((cols+1,1+nlags*cols))
        elif const_term == 'ct':
            betta = numpy.zeros((cols+2,2+nlags*cols))
        elif const_term == 'ctt':
            betta = numpy.zeros((cols+3,3+nlags*cols))
        if const_term == 'cc' or const_term == 'tt': fcols = cols+1
        elif const_term == 'ct': fcols = cols+2
        elif const_term == 'ctt': fcols = cols+3
        elif const_term == None: fcols = cols
        for vari in range(0,fcols,1):
            ymat=matd[nlags:,vari]
            Xy = numpy.dot(fmatd.T,ymat)
            betta[vari,:] = numpy.dot(XXi,Xy)
        if smbetta:
            # Produce the betta format from statsmodels library so we can use some of their code
            # Exclude the trend or constant from bettasm, as opposed to in betta
            bettasm = numpy.zeros((nlags,cols,cols))
            for lago in range(0,nlags,1):
                for varos in range(0,cols,1):
                    if const_term == 'tt' or const_term == 'cc':
                        bettasm[lago][varos,:] = betta[:,1:][varos][cols*lago:cols*(lago+1)]
                    elif const_term == 'ct':
                        bettasm[lago][varos,:] = betta[:,2:][varos][cols*lago:cols*(lago+1)]
                    elif const_term == 'ctt':
                        bettasm[lago][varos,:] = betta[:,3:][varos][cols*lago:cols*(lago+1)]
                    elif const_term == None:
                        bettasm[lago][varos,:] = betta[varos][cols*lago:cols*(lago+1)]
        if not func:
            if smbetta:
                self.betta_one_sm = bettasm
                self.betta_one = betta
            elif not smbetta:
                self.betta_one = betta
        elif func:
            if smbetta:
                return bettasm
            elif not smbetta:
                return betta

            
    def mkCompMat(self,betta=None,betta_one=None,func=False):
        if betta == None: betta = copy.deepcopy(self.betta)
        if betta_one == None and 'betta_one' in dir(self):
            betta_one = copy.deepcopy(self.betta_one)
        cols = self.ncols
        lags = self.nlags
        const = self.confdic['const_term']
        if const == None:
            compmat = numpy.zeros((cols*lags,cols*lags))
            eyem = numpy.eye(cols)
            compmat[:cols,:] = betta
            for lago in range(1,lags,1):
                compmat[cols*lago:cols*(lago+1),cols*(lago-1):cols*lago] = eyem
        elif const == 'cc' or const == 'tt':
            compmat = numpy.zeros((1+cols*lags,1+cols*lags))
            compmat[0,0] = 1.0
            compmat[0,1:] = 0.0
            compmat[1:,0] = 0.0
            eyem = numpy.eye(cols)
            compmat[1:cols+1,:] = betta_one[1:,:]
            for lago in range(1,lags,1):
                compmat[1+cols*lago:cols*(lago+1)+1,cols*(lago-1):cols*lago] = eyem
        elif const == 'ct':
            compmat = numpy.zeros((2+cols*lags,2+cols*lags))
            compmat[0,0] = 1.0
            compmat[1,1] = 1.0
            compmat[0,2:] = 0.0
            compmat[2:,0] = 0.0
            eyem = numpy.eye(cols)
            compmat[2:cols+2,:] = betta_one[2:,:]
            for lago in range(1,lags,1):
                compmat[2+cols*lago:cols*(lago+1)+2,cols*(lago-1):cols*lago] = eyem
        elif const == 'ctt':
            compmat = numpy.zeros((3+cols*lags,3+cols*lags))
            compmat[0,0] = 1.0
            compmat[1,1] = 1.0
            compmat[0,3:] = 0.0
            compmat[3:,0] = 0.0
            eyem = numpy.eye(cols)
            compmat[3:cols+3,:] = betta_one[3:,:]
            for lago in range(1,lags,1):
                compmat[3+cols*lago:cols*(lago+1)+3,cols*(lago-1):cols*lago] = eyem
        if not func:
            self.compmat = copy.deepcopy(compmat)
            # Also create a companion matrix whic strips out the constant and time trend effects, only focus on short-run behaviour
            if const == None:
                self.compmatr = compmat
            elif const == 'cc' or const == 'tt':
                self.compmatr = compmat[1:,1:]
                self.compmatr[cols:,:] = 0.0
                for lago in range(1,lags,1):
                    self.compmatr[cols*lago:cols*(lago+1),cols*(lago-1):cols*lago] = eyem             
            elif const == 'ct':
                self.compmatr = compmat[2:,2:]
                self.compmatr[cols:,:] = 0.0
                for lago in range(1,lags,1):
                    self.compmatr[cols*lago:cols*(lago+1),cols*(lago-1):cols*lago] = eyem
            elif const == 'ctt':
                self.compmatr = compmat[3:,3:]
                self.compmatr[cols:,:] = 0.0
                for lago in range(1,lags,1):
                    self.compmatr[cols*lago:cols*(lago+1),cols*(lago-1):cols*lago] = eyem
        elif func:
            # Also create a companion matrix whic strips out the constant and time trend effects, only focus on short-run behaviour
            if const == None:
                compmatr = compmat
            elif const == 'cc' or const == 'tt':
                compmatr = compmat[1:,1:]
                compmatr[cols:,:] = 0.0
                for lago in range(1,lags,1):
                    compmatr[cols*lago:cols*(lago+1),cols*(lago-1):cols*lago] = eyem                            
            elif const == 'ct':
                compmatr = compmat[2:,2:]
                compmatr[cols:,:] = 0.0
                for lago in range(1,lags,1):
                    compmatr[cols*lago:cols*(lago+1),cols*(lago-1):cols*lago] = eyem
            elif const == 'ctt':
                compmatr = compmat[3:,3:]
                compmatr[cols:,:] = 0.0
                for lago in range(1,lags,1):
                    compmatr[cols*lago:cols*(lago+1),cols*(lago-1):cols*lago] = eyem
            return compmatr

    def mkfitres(self,matd=None,nmatd=None,betta=None,nlags=None,func=False):
        # Create fitted values and residuals
        if matd == None: matd = self.data
        if nmatd == None: nmatd = self.nmatd
        if betta == None: betta = self.betta
        if nlags == None: nlags = self.nlags
        yfmat = numpy.dot(nmatd,betta.T)
        resmat = matd[nlags:]-yfmat
        covmat = numpy.cov(resmat.T)
        if not func:
            self.yfmat = yfmat
            self.resmat = resmat
            self.covmat = covmat
        elif func:
            return yfmat,resmat,covmat
    
    def mksigma(self,resmat=None,func=False):
        if resmat == None: resmat = self.resmat
        vnames = self.vnames
        svnames = self.svnames
        transd = self.tcode_dic
        # Create the variance-covariance matrix of the residuals
        vcmat = numpy.cov(resmat.T)
        # Create the Cholesky decomposition of vcmat
        cholmat = numpy.linalg.cholesky(vcmat)
        # This is the step which scales up the initial shock, if so configured in svname
        eyem = numpy.eye(cholmat.shape[0])
        if self.confdic['use_svnames']:
            for i1,name in enumerate(vnames):
                if transd[name] in [4,5,6,7,8]:
                    if svnames[name] != 1.0:
                        adjo = numpy.log(float(svnames[name]))/float(cholmat[i1,i1])
                        eyem[i1,i1] = eyem[i1,i1]*adjo
                    else: eyem[i1,i1] = 1.0
                else:
                    if svnames[name] != 1.0:
                        adjo = float(svnames[name])/float(cholmat[i1,i1])
                        eyem[i1,i1] = eyem[i1,i1]*adjo
                    else:
                        eyem[i1,i1] = 1.0
        cholmat_scaled = numpy.dot(cholmat,eyem)
        if not func:
            self.vcmat = vcmat
            self.cholmat_scaled = cholmat_scaled
            self.cholmat = cholmat_scaled
            self.cholmat_unscaled = cholmat
            self.eyem = eyem
        elif func:
            return vcmat,cholmat_scaled,eyem

    def mkphis(self,bettasm=None,betta=None,compmatr=None,maxphi=None,cholmat=None,cholimp=None,func=False):
        if bettasm == None: bettasm = self.bettasm
        if maxphi == None: maxphi = self.confdic['maxphi']
        if cholimp == None : cholimp = self.confdic['cholimp']
        if cholmat == None: cholmat = self.cholmat
        if compmatr == None: compmatr = copy.deepcopy(self.compmatr)
        if betta == None: betta = copy.deepcopy(self.betta)
        nlags = self.nlags
        ncols = self.ncols
        const_term = self.confdic['const_term']
        bettar = betta[:,-ncols*nlags:]
        loglist = self.confdic['loglist']
        vnames = self.vnames
        const = self.confdic['const_term']
        k = copy.deepcopy(self.ncols)
        p = copy.deepcopy(self.nlags)
        transd = self.trans_dic
        # Now try the version using the companion matrix
        betta_comp = compmatr
        phis_comp = numpy.zeros((maxphi+1,k,k))
        bigmat_temp = copy.deepcopy(betta_comp)
        phis_comp[0] = numpy.eye(k)       
        # recursively compute Phi matrices, but using the first-order companion matrix
        for i in xrange(1, maxphi + 1):
            '''
            phis_comp[i] = bigmat_temp[:k,:k]
            bigmat_temp = numpy.dot(bigmat_temp,betta_comp)
            '''
            for j in xrange(1,i+1):
                if j > nlags: break
                phis_comp[i] += numpy.dot(phis_comp[i-j],bettar[:,(j-1)*k:j*k])
        self.phis_comp = copy.deepcopy(phis_comp)
        # Also create the orthogonalized equivalents
        if cholimp:
            phisc = numpy.array([list(numpy.dot(tmat, cholmat)) for tmat in phis_comp[1:]])
            phisc[0] = numpy.eye(phisc.shape[2])
            for i1,name in enumerate(vnames):
                if transd[name] in [4,5,6,7,8]:
                    phisc[0][i1,i1] = 0.01
        else:
            phisc = numpy.array([list(numpy.dot(tmat, cholmat)) for tmat in phis_comp])
        if not func:
            self.phis = copy.deepcopy(phis_comp)
            self.phisc = copy.deepcopy(phisc)
        elif func:
            return phis_comp,phisc

    def mkboot(self,bdraw=None,yfmat=None,resiarray=None):
        # Get all of the paramters
        if bdraw == None: bdraw = self.confdic['bdraw']
        maxphi = self.confdic['maxphi']
        signif = self.confdic['signif']
        irfp = self.confdic['irfp']
        if resiarray == None: resmat_n = copy.deepcopy(self.resmat)
        # Need to centre residuals prior to using them in the bootstrap procedure
        resmat_n = resmat_n - numpy.mean(resmat_n,axis=0)        
        if yfmat == None: yfmat = self.yfmat
        nlags = copy.deepcopy(self.nlags)
        nrows = copy.deepcopy(self.nrows)
        ncols = copy.deepcopy(self.ncols)
        matd = copy.deepcopy(self.data)
        const = self.confdic['const_term']
        shifto = 0
        if const == 'cc' or const == 'tt':
            shifto = 1
        elif const == 'ct':
            shifto = 2
        elif const == 'ctt':
            shifto = 3
        betta = copy.deepcopy(self.betta)

        # Produce bootstrap confidence intervals
        barray = []
        betarray = []
        rarray = []
        resarray = []
        nmatarray = []
        # Do the first bootstrap run a la Killian 1998 to reduce the small sample bias of the IRFs, if wanted
        if self.confdic['killian_bsinbs']:
            self.phis_uncorr = copy.deepcopy(self.phis)
            print "Calculating bias correction using Killian's(1998) bs-in-bs method"
            kill_array = []
            for bodraw in range(0,bdraw,1):
                init_vals = self.data[:nlags,:]
                matd_n = numpy.zeros((nrows,ncols))
                matd_n[:nlags,:] = copy.deepcopy(init_vals)                
                numpy.random.shuffle(resmat_n)
                for obso in xrange(0,(nrows-nlags),1):
                    for j in xrange(1,nlags+1):
                        # Producing the fitted values period-by-period
                        if const == None:
                            matd_n[nlags+obso,:] += numpy.dot(matd_n[nlags+obso-j,:],betta[:,(j-1)*ncols:j*ncols].T)
                        elif const == 'cc':
                            matd_n[nlags+obso,:] += numpy.dot(matd_n[nlags+obso-j,:],betta[:,1+(j-1)*ncols:1+j*ncols].T)
                        elif const == 'tt':
                            matd_n[nlags+obso,:] += numpy.dot(matd_n[nlags+obso-j,:],betta[:,1+(j-1)*ncols:1+j*ncols].T)+betta[:,0].T
                        elif const == 'ct':
                            matd_n[nlags+obso,:] += numpy.dot(matd_n[nlags+obso-j,:],betta[:,2+(j-1)*ncols:2+j*ncols].T)+betta[:,1].T                    
                    # Then finally adding the reshuffled residuals period-by-period
                    matd_n[nlags+obso,:] += resmat_n[obso,:]
                nmatd_n = genXX(self,matd=matd_n,func=True)
                betta_n = compbetta(self,matd=matd_n,nmatd=nmatd_n,smbetta=False,func=True)
                kill_array.append(betta_n)
            kill_array = numpy.array(kill_array)
            biasred_betta = numpy.mean(kill_array,axis=0)
            # Now calculated bias-corrected matrix of estimated coefficients and build alternative yfmat
            biasred_term = biasred_betta - self.betta
            # Make the bias in value and percentage available for inspection
            self.killbias = copy.deepcopy(biasred_term)
            self.killbiasperc = copy.deepcopy((biasred_term/self.betta)*100.0)
            self.betta_kcorr = copy.deepcopy(self.betta + biasred_term)
            betta_corrected = self.betta + biasred_term
            '''
            nmatd = copy.deepcopy(self.nmatd)
            yfmat = numpy.dot(nmatd,betta_corrected.T)
            resmat_n = matd[nlags:,:] - yfmat
            # Need to centre residuals prior to using them in the bootstrap procedure
            resmat_n = resmat_n - numpy.mean(resmat_n,axis=0)
            '''
            self.betta_uncorr = copy.deepcopy(self.betta)
            betta = copy.deepcopy(self.betta_kcorr)
            if 'betta_one' in dir(self):
                self.betta_one_uncorr = copy.deepcopy(self.betta_one)
                betta_one_kcorr = copy.deepcopy(self.betta_one)
                if self.confdic['const_term'] == 'cc' or self.confdic['const_term'] == 'tt':
                    betta_one_kcorr[1:,:] = copy.deepcopy(self.betta_kcorr)
                elif self.confdic['const_term'] == 'ct':
                    betta_one_kcorr[2:,:] = copy.deepcopy(self.betta_kcorr)
                elif self.confdic['const_term'] == 'ctt':
                    betta_one_kcorr[3:,:] = copy.deepcopy(self.betta_kcorr)
                compmatr = self.mkCompMat(betta=self.betta_kcorr,betta_one=betta_one_kcorr,func=True)
            else:
                compmatr = self.mkCompMat(betta=self.betta_kcorr,func=True)
            self.phisc_uncorr = copy.deepcopy(self.phisc)
            phis_kcorr,phisc_kcorr = self.mkphis(betta=self.betta_kcorr,compmatr=compmatr,func=True)
            self.phisc_kcorr = copy.deepcopy(phisc_kcorr)
            self.phis_kcorr = copy.deepcopy(phis_kcorr)
            self.phisc_uncorr = copy.deepcopy(self.phisc)
            self.phisc = copy.deepcopy(self.phisc_kcorr)     
            print 'Calculation of bias correction done...'
        print 'Now computing bootstraps'            
        for bodraw in range(0,bdraw,1):
            numpy.random.shuffle(resmat_n)
            init_vals = self.data[:nlags,:]
            matd_n = numpy.zeros((nrows,ncols))
            matd_n[:nlags,:] = copy.deepcopy(init_vals)
            for obso in xrange(0,(nrows-nlags),1):
                for j in xrange(1,nlags+1):
                    if const == None:
                        matd_n[nlags+obso,:] += numpy.dot(matd_n[nlags+obso-j,:],betta[:,(j-1)*ncols:j*ncols].T)
                    elif const == 'cc':
                        matd_n[nlags+obso,:] += numpy.dot(matd_n[nlags+obso-j,:],betta[:,1+(j-1)*ncols:1+j*ncols].T)
                    elif const == 'tt':
                        matd_n[nlags+obso,:] += numpy.dot(matd_n[nlags+obso-j,:],betta[:,1+(j-1)*ncols:1+j*ncols].T)+betta[:,0].T
                    elif const == 'ct':
                        matd_n[nlags+obso,:] += numpy.dot(matd_n[nlags+obso-j,:],betta[:,2+(j-1)*ncols:2+j*ncols].T)+betta[:,1].T
                matd_n[nlags+obso,:] += resmat_n[obso,:]
            rows_n,cols_n = matd_n.shape
            nmatd_n = genXX(self,matd=matd_n,func=True)
            betta_n = compbetta(self,matd=matd_n,nmatd=nmatd_n,smbetta=False,func=True)
            betta_one_n = self.compbettax(matd=matd_n,nmatd=nmatd_n,smbetta=False,func=True)
            compmatr_n = self.mkCompMat(betta=betta_n,betta_one=betta_one_n,func=True)
            # Use different names here as we don't want to overwrite the original resmat_n
            yfmat_nn,resmat_nn,covmat_nn = self.mkfitres(matd=matd_n,nmatd=nmatd_n,betta=betta_n,func=True)
            vcmat_nn,cholmat_nn,eyem_nn = self.mksigma(resmat=resmat_nn,func=True)
            # Use the original cholmat
            cholmat_n = copy.deepcopy(self.cholmat)
            phis_n,phisc_n = self.mkphis(compmatr=compmatr_n,betta=betta_n,maxphi=maxphi,cholmat=cholmat_n,func=True)
            barray.append(phisc_n)
        print 'Calculation of bootstraps done...'
        nmatarray = numpy.array(nmatarray)
        resarray = numpy.array(resarray)
        barray = numpy.array(barray)
        betarray = numpy.array(betarray)
        rarray = numpy.array(rarray)
        if type(signif) == type(10.0):
            lower = scipy.stats.scoreatpercentile(barray,per=((100.0-signif*100.0)/2.0))
            upper = scipy.stats.scoreatpercentile(barray,per=signif*100.0+((100.0-signif*100.0)/2.0))
            confint = [lower,upper]
        elif type(signif) == type([10.0,10.0]):
            lower_outer = scipy.stats.scoreatpercentile(barray,per=(100.0-signif[1]*100.0)/2.0)
            lower_inner = scipy.stats.scoreatpercentile(barray,per=(100.0-signif[0]*100.0)/2.0)
            upper_inner = scipy.stats.scoreatpercentile(barray,per=signif[0]*100.0+((100.0-signif[0]*100.0)/2.0))
            upper_outer = scipy.stats.scoreatpercentile(barray,per=signif[1]*100.0+((100.0-signif[1]*100.0)/2.0))
            confint = [lower_outer,lower_inner,upper_inner,upper_outer]
        self.rarray = rarray
        self.betarray = betarray
        self.barray = barray
        self.confint = numpy.array(confint)[:,:irfp+1]
        self.resarray = resarray
        self.nmatarray = nmatarray
        # Switch out of boot mode
        self.boot = False

    def refac_killian_pp(self,intup):
        resmat = intup[0]
        yfmat = intup[1]
        matd = intup[2]
        nlags = intup[3]
        maxphi = intup[4]
        betta = intup[5]
        nrows = self.nrows
        ncols = self.ncols
        nlags = self.nlags
        const = self.const_term
        resmat_n = copy.deepcopy(resmat)
        numpy.random.shuffle(resmat_n)
        init_vals = self.data[:nlags,:]
        matd_n = numpy.zeros((nrows,ncols))
        matd_n[:nlags,:] = copy.deepcopy(init_vals)
        numpy.random.shuffle(resmat_n)
        for obso in xrange(0,(nrows-nlags),1):
            for j in xrange(1,nlags+1):
                if const == None:
                    matd_n[nlags+obso,:] += numpy.dot(matd_n[nlags+obso-j,:],betta[:,(j-1)*ncols:j*ncols].T)
                elif const == 'cc':
                    matd_n[nlags+obso,:] += numpy.dot(matd_n[nlags+obso-j,:],betta[:,1+(j-1)*ncols:1+j*ncols].T)
                elif const == 'tt':
                    matd_n[nlags+obso,:] += numpy.dot(matd_n[nlags+obso-j,:],betta[:,1+(j-1)*ncols:1+j*ncols].T)+betta[:,0].T
                elif const == 'ct':
                    matd_n[nlags+obso,:] += numpy.dot(matd_n[nlags+obso-j,:],betta[:,2+(j-1)*ncols:2+j*ncols].T)+betta[:,1].T                    
            matd_n[nlags+obso,:] += resmat_n[obso,:]
        nmatd_n = pymaclab.stats.common.genXX(self,matd=matd_n,func=True)
        betta_n = pymaclab.stats.common.compbetta(self,matd=matd_n,nmatd=nmatd_n,smbetta=False,func=True)
        compbetta_n = self.mkCompMat(betta=betta_n,func=True)
        eigs = numpy.linalg.eigvals(compbetta_n)
        sbool = (numpy.abs(eigs) <= 1).all()
        if not sbool: print 'NOT STABLE !'
        return betta_n

    def refac_pp(self,intup):
        resmat = intup[0]
        yfmat = intup[1]
        matd = intup[2]
        nlags = intup[3]
        maxphi = intup[4]
        betta = intup[5]
        const = self.const_term
        nlags = self.nlags
        nrows = self.nrows
        ncols = self.ncols
        resmat_n = copy.deepcopy(resmat)
        numpy.random.shuffle(resmat_n)
        init_vals = self.data[:nlags,:]
        matd_n = numpy.zeros((nrows,ncols))
        matd_n[:nlags,:] = copy.deepcopy(init_vals)
        for obso in xrange(0,(nrows-nlags),1):
            for j in xrange(1,nlags+1):
                if const == None:
                    matd_n[nlags+obso,:] += numpy.dot(matd_n[nlags+obso-j,:],betta[:,(j-1)*ncols:j*ncols].T)
                elif const == 'cc':
                    matd_n[nlags+obso,:] += numpy.dot(matd_n[nlags+obso-j,:],betta[:,1+(j-1)*ncols:1+j*ncols].T)
                elif const == 'tt':
                    matd_n[nlags+obso,:] += numpy.dot(matd_n[nlags+obso-j,:],betta[:,1+(j-1)*ncols:1+j*ncols].T)+betta[:,0].T
                elif const == 'ct':
                    matd_n[nlags+obso,:] += numpy.dot(matd_n[nlags+obso-j,:],betta[:,2+(j-1)*ncols:2+j*ncols].T)+betta[:,1].T                    
            matd_n[nlags+obso,:] += resmat_n[obso,:]
        nmatd_n = pymaclab.stats.common.genXX(self,matd=matd_n,func=True)
        betta_n = pymaclab.stats.common.compbetta(self,matd=matd_n,nmatd=nmatd_n,smbetta=False,func=True)
        betta_one_n = self.compbettax(matd=matd_n,nmatd=nmatd_n,smbetta=False,func=True)
        compmatr_n = self.mkCompMat(betta=betta_n,betta_one=betta_one_n,func=True)
        # Lets try here to actually keep the Cholesky shock matrix fixed at the original one...
        yfmat_n,resmat_n,covmat_n = self.mkfitres(matd=matd_n,nmatd=nmatd_n,betta=betta_n,func=True)
        vcmat_n,cholmat_n,eyem_n = self.mksigma(resmat=resmat_n,func=True)
        cholmat_n = copy.deepcopy(self.cholmat)
        phis_n,phisc_n = self.mkphis(compmatr=compmatr_n,betta=betta_n,maxphi=maxphi,cholmat=cholmat_n,func=True)
        return phisc_n

    def mkboot_pp(self,bdraw=None,yfmat=None,resiarray=None):
        # Set instance into boot mode
        self.boot = True
        # Get all of the paramters
        if bdraw == None: bdraw = self.confdic['bdraw']
        maxphi = self.confdic['maxphi']
        signif = self.confdic['signif']
        irfp = self.confdic['irfp']
        const_term = copy.deepcopy(self.confdic['const_term'])
        betta = copy.deepcopy(self.betta)
        resmat = copy.deepcopy(self.resmat)
        ncols = copy.deepcopy(self.ncols)
        nlags = copy.deepcopy(self.nlags)
        # Need to centre the residuals prior to using them in boostrapping
        resmat = resmat - numpy.mean(resmat,axis=0)
        if yfmat == None: yfmat = copy.deepcopy(self.yfmat)
        matd = copy.deepcopy(self.data)
        # Produce bootstrap confidence intervals, here using parallel python to speed up computation
        barray = []
        ppservers = ()
        if self.confdic['ncpus'] == 'auto':
            job_server = pp.Server(ncpus='autodetect',ppservers=ppservers)
        else:
            job_server = pp.Server(ncpus=self.confdic['ncpus'],ppservers=ppservers)
        # Do the first bootstrap run a la Killian 1998 to reduce the small sample bias of the IRFs, if wanted
        if self.confdic['killian_bsinbs']:
            self.phis_uncorr = copy.deepcopy(self.phis)
            print "Calculating bias correction using Killian's(1998) bs-in-bs method"
            barray_biasred = []
            # Note: Only pass the betta coefficients of the dynamic responses, not constant or time-trend
            inputs_biasred = ((resmat,yfmat,matd,nlags,maxphi,betta),)*bdraw
            barray_biasred_jobs = [job_server.submit(self.refac_killian_pp,(input,),modules=("copy","numpy","scipy","pymaclab.stats.common",)) for input in inputs_biasred]
            barray_biasred = [x() for x in barray_biasred_jobs]
            barray_biasred = numpy.array(barray_biasred)
            biasred_term = numpy.mean(barray_biasred-self.betta,axis=0)
            # Make the bias in value and percentage available for inspection
            self.killbias = copy.deepcopy(biasred_term)
            self.killbiasperc = copy.deepcopy((biasred_term/self.betta)*100.0)
            self.betta_kcorr = copy.deepcopy(self.betta + biasred_term)
            betta_corrected = self.betta + biasred_term
            # Pass the corrected betta on to the boostrap procedure
            betta = copy.deepcopy(betta_corrected)
            self.betta_uncorr = copy.deepcopy(self.betta)
            if 'betta_one' in dir(self):
                self.betta_one_uncorr = copy.deepcopy(self.betta_one)
                betta_one_kcorr = copy.deepcopy(self.betta_one)
                if self.confdic['const_term'] == 'cc' or self.confdic['const_term'] == 'tt':
                    betta_one_kcorr[1:,:] = copy.deepcopy(self.betta_kcorr)
                elif self.confdic['const_term'] == 'ct':
                    betta_one_kcorr[2:,:] = copy.deepcopy(self.betta_kcorr)
                elif self.confdic['const_term'] == 'ctt':
                    betta_one_kcorr[3:,:] = copy.deepcopy(self.betta_kcorr)
                compmatr = self.mkCompMat(betta=self.betta_kcorr,betta_one=betta_one_kcorr,func=True)
            else:
                compmatr = self.mkCompMat(betta=self.betta_kcorr,func=True)
            self.phisc_uncorr = copy.deepcopy(self.phisc)
            # Recalculation with new betta
            #yfmat,resmat,covmat = self.mkfitres(betta=self.betta_kcorr,func=True)
            #self.covmat_kcorr = copy.deepcopy(covmat)
            #self.covmat = copy.deepcopy(self.covmat_kcorr)
            #self.resmat_kcorr = copy.deepcopy(resmat)
            #self.resmat = copy.deepcopy(self.resmat_kcorr)
            #self.yfmat_kcorr = copy.deepcopy(yfmat)
            #self.yfmat = copy.deepcopy(self.yfmat_kcorr)
            # Needs to be called before new impulses are created, because colmat needs to be updated
            #self.cholmat_kcorr = self.mksigma(resmat=self.resmat_kcorr,func=True)[1]
            #self.cholmat_uncorr = copy.deepcopy(self.cholmat)
            #self.cholmat = copy.deepcopy(self.cholmat_kcorr)
            phis_kcorr,phisc_kcorr = self.mkphis(betta=self.betta_kcorr,compmatr=compmatr,func=True)
            self.phisc_kcorr = copy.deepcopy(phisc_kcorr)
            self.phis_kcorr = copy.deepcopy(phis_kcorr)
            self.phisc_uncorr = copy.deepcopy(self.phisc)
            self.phisc = copy.deepcopy(self.phisc_kcorr)
            print 'Calculation of bias correction done...'
        print 'Now computing bootstraps'
        # Note: Again, pass only the betta coefficients for dynamic responses, no constant or time-trend
        inputs = ((resmat,yfmat,matd,nlags,maxphi,betta),)*bdraw
        barray_jobs = [job_server.submit(self.refac_pp,(input,),modules=("copy","numpy","scipy","pymaclab.stats.common")) for input in inputs]
        barray = [x() for x in barray_jobs]
        print 'Calculation of bootstraps done...'
        barray = numpy.array(barray)
        if type(signif) == type(10.0):
            lower = scipy.stats.scoreatpercentile(barray,per=((100.0-signif*100.0)/2.0))
            upper = scipy.stats.scoreatpercentile(barray,per=signif*100.0+((100.0-signif*100.0)/2.0))
            confint = [lower,upper]
        elif type(signif) == type([10.0,10.0]):
            lower_outer = scipy.stats.scoreatpercentile(barray,per=(100.0-signif[1]*100.0)/2.0)
            lower_inner = scipy.stats.scoreatpercentile(barray,per=(100.0-signif[0]*100.0)/2.0)
            upper_inner = scipy.stats.scoreatpercentile(barray,per=signif[0]*100.0+((100.0-signif[0]*100.0)/2.0))
            upper_outer = scipy.stats.scoreatpercentile(barray,per=signif[1]*100.0+((100.0-signif[1]*100.0)/2.0))
            confint = [lower_outer,lower_inner,upper_inner,upper_outer]
        self.barray = barray
        self.confint = numpy.array(confint)[:,:irfp+1]
        # Switch out of boot mode
        self.boot = False
        
    def genFEVD(self,y,coefs,steps,func=False):
        p = len(coefs)
        k = len(coefs[0])
        # initial value
        forcs = numpy.zeros((steps, k))
    
        # h=0 forecast should be latest observation
        # forcs[0] = y[-1]
    
        # make indices easier to think about
        for h in xrange(1, steps + 1):
            # y_t(h) = intercept + sum_1^p A_i y_t_(h-i)
            f = forcs[h - 1]
            for i in xrange(1, p + 1):
                # slightly hackish
                if h - i <= 0:
                    # e.g. when h=1, h-1 = 0, which is y[-1]
                    prior_y = y[h - i - 1]
                else:
                    # e.g. when h=2, h-1=1, which is forcs[0]
                    prior_y = forcs[h - i - 1]
    
                # i=1 is coefs[0]
                f = f + numpy.dot(coefs[i - 1], prior_y)
            forcs[h - 1] = f
            
        self.forcs = copy.deepcopy(forcs)

        if 'phis_kcorr' in dir(self): ma_coefs = copy.deepcopy(self.phis_kcorr[:steps])
        else: ma_coefs = copy.deepcopy(self.phis[:steps])
        sigma_u = copy.deepcopy(self.covmat)
        k = len(sigma_u)
        forc_covs = numpy.zeros((steps, k, k))

        prior = numpy.zeros((k, k))
        for h in xrange(steps):
            # Sigma(h) = Sigma(h-1) + Phi Sig_u Phi'
            phi = ma_coefs[h]
            var = numpy.dot(numpy.dot(phi,sigma_u),phi.T)
            forc_covs[h] = prior = prior + var
            
        self.forc_covs = copy.deepcopy(forc_covs)
            
        periods = steps
        neqs = self.data.shape[1]
        vnames = self.vnames
        
        if 'phisc_kcorr' in dir(self): orthirfs = copy.deepcopy(self.phisc_kcorr)
        else: orthirfs = copy.deepcopy(self.phisc)

        # cumulative impulse responses
        irfs = (orthirfs[:periods] ** 2).cumsum(axis=0)
        self.forc_covs_irfs = copy.deepcopy(irfs)

        rng = range(neqs)
        mse = forc_covs[:, rng, rng]

        # lag x equation x component
        fevd = numpy.empty_like(irfs)

        for i in range(periods):
            fevd[i] = (irfs[i].T / mse[i]).T
        
        # Express in percentage terms
        fevd = fevd*100.0

        # switch to equation x lag x component
        self.decomp = fevd.swapaxes(0, 1)


    def plot_vals(self):
        # Graph some vars and dump them in graphs directory
        vnames = self.vnames
        matd = self.data
        nlags = self.nlags
        modname = self.confdic['modname']
        if modname not in os.listdir('../graphs/'):
            os.mkdir('../graphs/'+modname)
        yfmat = self.yfmat
        resmat = self.resmat
        tcode_dic = self.confdic['tcode']
        for col,name in enumerate(vnames):
            fig1 = plt.figure()
            ax1 = fig1.add_subplot(3,1,1)
            ax1.grid()
            if tcode_dic[name] in [4,5,6,7,8,16]:
                ax1.plot(matd[nlags:,col]*100.0,color='black')
            else:
                ax1.plot(matd[nlags:,col],color='black')
            ax1.set_title('Actual Values for '+name)
            ax2 = fig1.add_subplot(3,1,2)
            ax2.grid()
            if tcode_dic[name] in [4,5,6,7,8,16]:
                ax2.plot(yfmat[:,col]*100.0,color='black')
            else:
                ax2.plot(yfmat[:,col],color='black')
            ax2.set_title('Fitted Values for '+name)
            ax3 = fig1.add_subplot(3,1,3)
            ax3.grid()
            if tcode_dic[name] in [4,5,6,7,8,16]:
                ax3.plot(resmat[:,col]*100.0,color='black')
            else:
                ax3.plot(resmat[:,col],color='black')
            ax3.set_title('Residuals for '+name)
            if self.confdic['graph_options']['save']['format']['eps']:
                fig1.savefig('../graphs/'+modname+'/'+name+'.eps',bbox_inches='tight')
            if self.confdic['graph_options']['save']['format']['pdf']:
                fig1.savefig('../graphs/'+modname+'/'+name+'.pdf',bbox_inches='tight')
        plt.close('all')

    def plot_irfs(self):        
        # Graph some IRFs (Cholesky)
        irf_options = self.confdic['graph_options']['irfs']
        outer_colour = irf_options['outer_colour']
        inner_colour = irf_options['inner_colour']
        line_colour = irf_options['line_colour']
        glopt = self.confdic['graph_options']['labels']
        gridopt = self.confdic['graph_options']['grid']
        vnames = self.vnames
        pnames = self.pnames
        irfp = self.confdic['irfp']
        cols = self.ncols
        phisc = self.phisc
        confint = self.confint
        loglist = self.confdic['loglist']
        tcode_dic = self.confdic['tcode']
        translog = self.confdic['translog']
        transperc = self.confdic['transperc']
        transperc_dic = self.confdic['transperc_dic']
        transdic = self.trans_dic
        hzline = self.confdic['hzline']
        hzlwidth = self.confdic['hzline_width']
        bootstrap = self.confdic['bootstrap']
        kbsinbs = self.confdic['killian_bsinbs']
        if kbsinbs: phisc_uncorr = self.phisc_uncorr
        graph_biased = self.confdic['graph_options']['irfs']['biased_line']
        graph_biased_type = self.confdic['graph_options']['irfs']['biased_line_type']
        modname = self.confdic['modname']
        # Check if folder(s) needs to be created
        if modname not in os.listdir('../graphs/'):
            os.mkdir('../graphs/'+modname)
        if 'shock_irfs' not in os.listdir('../graphs/'+modname+'/'):
            os.mkdir('../graphs/'+modname+'/shock_irfs') 
        signif = self.confdic['signif']
        for shockv,sname in enumerate(vnames):
            fig1 = plt.figure()
            for respv,rname in enumerate(vnames):
                fig2 = plt.figure()
                ax2 = fig2.add_subplot(1,1,1)
                if glopt['xlabel']: ax2.set_xlabel('Time periods after shock',fontsize=glopt['xlabel_fs'])
                ax = fig1.add_subplot(cols,1,respv+1)
                if glopt['xlabel']: ax.set_xlabel('Time periods after shock',fontsize=glopt['xlabel_fs'])
                if tcode_dic[rname] in [4,5,6,7,8,16] and translog and not transperc_dic[rname]:
                    if glopt['ylabel']: ax.set_ylabel(r'Percentage Deviation',fontsize=glopt['ylabel_fs'])
                    if glopt['ylabel']: ax2.set_ylabel(r'Percentage Deviation',fontsize=glopt['ylabel_fs'])
                else:
                    if glopt['ylabel']: ax.set_ylabel(r'Absolute Deviation',fontsize=glopt['ylabel_fs'])
                    if glopt['ylabel']: ax2.set_ylabel(r'Absolute Deviation',fontsize=glopt['ylabel_fs'])
                if hzline:
                    ax2.hlines(numpy.array([0,]*irfp),xmin=0,xmax=irfp,linestyles='solid',linewidth=hzlwidth,colors='k')
                    ax.hlines(numpy.array([0,]*irfp),xmin=0,xmax=irfp,linestyles='solid',linewidth=hzlwidth,colors='k')
                if tcode_dic[rname] in [4,5,6,7,8,16] and translog and not transperc_dic[rname]:
                    percen = 100.0
                elif transperc and transperc_dic[rname]:
                    percen = numpy.exp(transdic[vnames[respv]]['mean'])
                else:
                    percen = 1.0
                # When using the non-parametric bootstrap the central (mean) IR has to be the average of all IRs
                if not bootstrap:
                    ax2.plot(phisc[:irfp+1,respv,shockv].flatten()*percen,color=line_colour)
                elif bootstrap:
                    ax2.plot(phisc[:irfp+1,respv,shockv].flatten()*percen,color=line_colour)
                if kbsinbs and graph_biased:
                    ax2.plot(phisc_uncorr[:irfp+1,respv,shockv].flatten()*percen,graph_biased_type)
                plt.xlim((0,irfp))
                if gridopt: ax2.grid()
                ax.plot(phisc[:irfp+1,respv,shockv].flatten()*percen,color=line_colour)
                if gridopt: ax.grid()
                if bootstrap:
                    if type(signif) == type(10.0):
                        ax2.fill_between([xx for xx in range(0,irfp+1,1)],confint[0,:,respv,shockv].flatten()*percen,confint[1,:,respv,shockv].flatten()*percen, facecolor=outer_colour, edgecolor=outer_colour)
                        ax.fill_between([xx for xx in range(0,irfp+1,1)],confint[0,:,respv,shockv].flatten()*percen,confint[1,:,respv,shockv].flatten()*percen, facecolor=outer_colour, edgecolor=outer_colour)
                    elif type(signif) == type([10.0,10.0]):
                        ax2.fill_between([xx for xx in range(0,irfp+1,1)],confint[0,:,respv,shockv].flatten()*percen,confint[1,:,respv,shockv].flatten()*percen, facecolor=inner_colour, edgecolor=inner_colour)
                        ax2.fill_between([xx for xx in range(0,irfp+1,1)],confint[1,:,respv,shockv].flatten()*percen,confint[2,:,respv,shockv].flatten()*percen, facecolor=outer_colour, edgecolor=outer_colour)
                        ax2.fill_between([xx for xx in range(0,irfp+1,1)],confint[2,:,respv,shockv].flatten()*percen,confint[3,:,respv,shockv].flatten()*percen, facecolor=inner_colour, edgecolor=inner_colour)
                        ax.fill_between([xx for xx in range(0,irfp+1,1)],confint[0,:,respv,shockv].flatten()*percen,confint[1,:,respv,shockv].flatten()*percen, facecolor=inner_colour, edgecolor=inner_colour)
                        ax.fill_between([xx for xx in range(0,irfp+1,1)],confint[1,:,respv,shockv].flatten()*percen,confint[2,:,respv,shockv].flatten()*percen, facecolor=outer_colour, edgecolor=outer_colour)
                        ax.fill_between([xx for xx in range(0,irfp+1,1)],confint[2,:,respv,shockv].flatten()*percen,confint[3,:,respv,shockv].flatten()*percen, facecolor=inner_colour, edgecolor=inner_colour)
                if glopt['title']: ax2.set_title('Response of '+pnames[rname]+' to shock on '+pnames[sname],fontsize=glopt['title_fs'])        
                if glopt['title']: ax.set_title('Response of '+pnames[rname]+' to shock on '+pnames[sname],fontsize=glopt['title_fs'])
                if self.confdic['graph_options']['save']['format']['eps']:
                    fig2.savefig('../graphs/'+modname+'/'+'shock_irfs/irf_'+sname+'_'+rname+'.eps',bbox_inches='tight')
                if self.confdic['graph_options']['save']['format']['pdf']:
                    fig2.savefig('../graphs/'+modname+'/'+'shock_irfs/irf_'+sname+'_'+rname+'.pdf',bbox_inches='tight')
                plt.close()
            if self.confdic['graph_options']['save']['format']['eps']:
                fig1.savefig('../graphs/'+modname+'/'+'irf_'+sname+'.eps',bbox_inches='tight')
            if self.confdic['graph_options']['save']['format']['pdf']:
                fig1.savefig('../graphs/'+modname+'/'+'irf_'+sname+'.pdf',bbox_inches='tight')
            plt.close()
        plt.close('all')


    def plot_one_irf(self,sname=None,rname=None,save_hd=True):       
        irf_options = self.confdic['graph_options']['irfs']
        outer_colour = irf_options['outer_colour']
        inner_colour = irf_options['inner_colour']
        line_colour = irf_options['line_colour']
        vnames = self.vnames
        shockv = vnames.index(sname)
        respv = vnames.index(rname)
        pnames = self.pnames
        irfp = self.confdic['irfp']
        cols = self.ncols
        phisc = self.phisc
        confint = self.confint
        loglist = self.confdic['loglist']
        tcode_dic = self.confdic['tcode']
        translog = self.confdic['translog']
        transperc = self.confdic['transperc']
        transperc_dic = self.confdic['transperc_dic']
        transdic = self.trans_dic
        hzline = self.confdic['hzline']
        bootstrap = self.confdic['bootstrap']
        modname = self.confdic['modname']
        # Check if folder(s) needs to be created
        if modname not in os.listdir('../graphs/'):
            os.mkdir('../graphs/'+modname)
        if 'shock_irfs' not in os.listdir('../graphs/'+modname+'/'):
            os.mkdir('../graphs/'+modname+'/shock_irfs') 
        signif = self.confdic['signif']
        fig = plt.figure()
        ax = fig.add_subplot(1,1,1)
        ax.set_xlabel('Lags')
        plt.xlim((0,irfp))
        ax.set_xlabel('Lags')
        plt.xlim((0,irfp))
        if tcode_dic[rname] in [4,5,6,7,8,16] and translog and not transperc_dic[rname]:
            ax.set_ylabel('\% Deviation')
        else:
            ax.set_ylabel('Abs. Deviation')
        if hzline:
            ax.hlines(numpy.array([0,]*irfp),xmin=0,xmax=irfp,linestyles='solid',linewidth=0.2,colors='k')
        if tcode_dic[rname] in [4,5,6,7,8,16] and translog and not transperc_dic[rname]:
            percen = 100.0
        elif transperc and transperc_dic[rname]:
            percen = numpy.exp(transdic[vnames[respv]]['mean'])
        else:
            percen = 1.0
            ax.plot(phisc[:irfp+1,respv,shockv].flatten()*percen,color=line_colour)
        ax.grid()
        if bootstrap:
            if type(signif) == type(10.0):
                ax.fill_between([xx for xx in range(0,irfp+1,1)],confint[0,:,respv,shockv].flatten()*percen,confint[1,:,respv,shockv].flatten()*percen, facecolor=outer_colour, edgecolor=outer_colour)
            elif type(signif) == type([10.0,10.0]):
                ax.fill_between([xx for xx in range(0,irfp+1,1)],confint[0,:,respv,shockv].flatten()*percen,confint[1,:,respv,shockv].flatten()*percen, facecolor=inner_colour, edgecolor=inner_colour)
                ax.fill_between([xx for xx in range(0,irfp+1,1)],confint[1,:,respv,shockv].flatten()*percen,confint[2,:,respv,shockv].flatten()*percen, facecolor=outer_colour, edgecolor=outer_colour)
                ax.fill_between([xx for xx in range(0,irfp+1,1)],confint[2,:,respv,shockv].flatten()*percen,confint[3,:,respv,shockv].flatten()*percen, facecolor=inner_colour, edgecolor=inner_colour)
        ax.set_title(r'$\textrm{Response of '+pnames[rname]+' to shock on '+pnames[sname]+'}$')        
        if save_hd:
            if self.confdic['graph_options']['save']['format']['eps']:
                fig.savefig('../graphs/'+modname+'/'+'shock_irfs/irf_'+sname+'_'+rname+'.eps',bbox_inches='tight')
            if self.confdic['graph_options']['save']['format']['pdf']:
                fig.savefig('../graphs/'+modname+'/'+'shock_irfs/irf_'+sname+'_'+rname+'.pdf',bbox_inches='tight')
        else:
            self.irf_graphs_dic[sname][rname] = fig     
        plt.close('all')


"""
This module implements efficient "low-level" number types for purely
numerical tasks.

We use the following types:

int/long -- (Python built-ins) exact integers
mpf -- arbitrary-precision floats
mpc -- arbitrary-precision complex floats
mpq -- nonintegral fractions
mpqc -- nonreal complex rationals

In the interest of speed, there are some quirks:

  * mpq derives from tuple. mpq.__init__ does *not* validate its input.
    mpqs *should only* be created from fully reduced (p, q) pairs,
    and p/q should *not* be integer-valued.

  * To safely create a fraction from two integers, use the function
    normalized_fraction()

  * Arithmetic operations on mpqs automatically normalize back to Python
    ints when the results are integer-valued. Likewise, mpqc with zero
    real part normalize to their real parts.

  * Note that ``mpq(2)/mpq(3)`` does *not* work as expected; it does the
    same thing as ``2/3`` in Python. To perform safe division, use the
    provided ``div`` function.

  * mpqs can be compared with Python ints, but cannot be (correctly)
    compared to Python floats. (Use the mpf class instead.)

Powers, except when the exponent is a positive integer, should be
computed with the ``try_power()`` function which detects when the
result is not exact.
"""
#
# Author: Fredrik Johansson
# Created: January 2008

import math

from . import mpmath

__docformat__ = "restructuredtext"

from ..core import init_module

init_module.import_heads()

@init_module
def _init(module):
    from ..basealgebra.verbatim import Verbatim
    module.Verbatim = Verbatim

from ..utils import str_SUM, str_PRODUCT, str_POWER, str_APPLY, str_SYMBOL, str_NUMBER, NUMBER, SYMBOL

inttypes = (int, long)

from .mpmath import mpf, mpc, mp
from .mpmath.libmp import from_rational, round_nearest



def mpf_to_str_data(self, sort=True):
    if self < 0:
        return str_SUM, str(self)
    return str_NUMBER, str(self)

def mpc_to_str_data(self, sort=True):
    return str_NUMBER, str(self)

mpf.to_str_data = mpf_to_str_data
mpc.to_str_data = mpc_to_str_data

rounding = round_nearest

def getdps():
    return mp.dps

def setdps(n):
    p = mp.dps
    mp.dps = int(n)
    return p

def getprec():
    return mp.prec

#----------------------------------------------------------------------------
# Fractions
#

def normalized_fraction(p, q=1):
    """ Return a normalized fraction.
    """
    x, y = p, q
    while y:
        x, y = y, x % y
    if x != 1:
        p //= x
        q //= x
    if q == 1:
        return p
    return mpq((p, q))

class mpq(tuple):
    """Represents a fraction."""

    # These methods are inherited directly from tuple for speed. This works
    # as long as all mpqs are normalized:
    # __new__/__init__
    # __nonzero__
    # __eq__
    # __hash__

    # These methods are generated and defined in methods.py:
    # __add__, __sub__, __rsub__, __mul__, __div__, __rdiv__, __pow__
    # __lt__, __le__, __gt__, __ge__

    __slots__ = []

    @property
    def _mpf_(self):
        p, q = self
        return from_rational(p, q, getprec(), rounding)

    def as_verbatim(self):
        p, q = self
        if p<0:
            return -(Verbatim(NUMBER, -p) / Verbatim(NUMBER, q))
        return Verbatim(NUMBER, p) / Verbatim(NUMBER, q)

    def to_str_data(self,sort=True):
        if self[0]<0:
            return str_SUM, str(self)
        return str_PRODUCT, str(self)

    def __str__(self):
        return "%i/%i" % self

    def __repr__(self):
        p, q = self
        return "%s((%s, %s))" % (type(self).__name__, p, q)

    def __float__(self):
        p, q = self
        return float(p) / q

    def __int__(self):
        p, q = self
        return p // q

    def __neg__(self):
        p, q = self
        return mpq((-p, q))

    def __pos__(self):
        return self

    def __abs__(self):
        p, q = self
        if p < 0:
            return mpq((-p, q))
        return self

    def __floordiv__(a, b):
        return int(a / b)

    def __rfloordiv__(a, b):
        return int(b / a)

    def __mod__(a, b):
        return a - (a//b)*b

    def __rmod__(a, b):
        return b - (b//a)*a

    def __divmod__(a, b):
        return (a-a%b)/b, a%b

    def __rdivmod__(a, b):
        return (b-b%a)/a, b%a

    def __rpow__(a, b):
        z, sym = try_power(b, a)
        if not sym:
            return z
        return NotImplemented


#----------------------------------------------------------------------------
# Complex numbers
#

def innerstr(x):
    if isinstance(x, mpq):
        return "%s/%s" % x
    return str(x)


class mpqc(object):
    """Represents an exact complex number.

    The integer power of a complex number is computed using
    `exponentiation by squaring`__ method.

    __ http://en.wikipedia.org/wiki/Exponentiation_by_squaring
    """

    __slots__ = ['real', 'imag']

    def __new__(cls, real, imag=0):
        if not imag:
            return real
        if isinstance(real, (float, mpf)) or isinstance(imag, (float, mpf)):
            return mpc(real, imag)
        self = object.__new__(mpqc)
        self.real = real
        self.imag = imag
        return self

    def __repr__(self):
        return "%s(%r, %r)" % (self.__class__.__name__, self.real, self.imag)

    def __hash__(self):
        return hash((self.real, self.imag))

    def __mpcval__(self):
        return mpc(self.real, self.imag)

    def __getstate__(self):
        return (self.real, self.imag)

    def __setstate__(self, (real, imag)):
        self.real = real
        self.imag = imag

    def to_str_data(self,sort=True):
        re, im = self.real, self.imag
        if re==0:
            if im == 1: return str_SYMBOL,"I"
            if im == -1: return str_SUM, "-I"
            return str_PRODUCT, str(self.imag) + "*I"
        restr = innerstr(self.real)
        if im == 1: return str_SUM, "%s + I" % restr
        if im == -1: return str_SUM, "%s - I" % restr
        if im > 0: return str_SUM, "%s + %s*I" % (restr, innerstr(self.imag))
        if im < 0: return str_SUM, "%s - %s*I" % (restr, innerstr(-self.imag))
        raise NotImplementedError(`self`)

    def __str__(self):
        return self.to_str_data()[1]

    def as_verbatim(self):
        re, im = self.real, self.imag
        if re < 0: 
            r = -Verbatim(NUMBER, -self.real)
        else:
            r = Verbatim(NUMBER, self.real)
        if im < 0:
            i = -Verbatim(NUMBER, -self.imag)
            ni = Verbatim(NUMBER, -self.imag)
        else:
            i = Verbatim(NUMBER, self.imag)
        I = Verbatim(NUMBER, mpqc(0,1))
        if re==0:
            if im == 1: return I
            if im == -1: return -I
            return i * I
        if im == 1: return r + I
        if im == -1: return r - I
        if im < 0:
            return r - ni * I
        else:
            return r + i * I

    def as_numer_denom(self):
        re, im = self.real, self.imag
        if type(re) is mpq:
            r1, r2 = re
        else:
            r1, r2 = re, 1
        if type(im) is mpq:
            i1, i2 = im
        else:
            i1, i2 = im, 1
        r = r1*i2
        i = i1*r2
        d = r2*i2
        c = gcd(r,i,d)
        return mpqc(r//c,i//c), d//c
            
    def __eq__(self, other):
        if hasattr(other, "imag"):
            return self.real == other.real and self.imag == other.imag
        return False

    def __pos__(self): return self
    def __neg__(self): return mpqc(-self.real, -self.imag)

    def __abs__(self):
        re, im = self.real, self.imag
        if not re:
            return abs(im)
        m2 = re**2 + im**2
        if isinstance(m2, inttypes):
            m, flag = int_root(m2,2)
            if flag:
                return m
        if isinstance(m2, mpq):
            p,fp = int_root(m2[0],2)
            q,fq = int_root(m2[1],2)
            if fp and fq:
                return mpq((p,q))
        raise NotImplementedError('abs(%r)' % (self))

#----------------------------------------------------------------------------
# Interface functions
#

inttypes = (int, long)
rationaltypes = (mpq,)
numbertypes = (int, long, float, complex, mpq, mpqc, mpf, mpc)
realtypes = (int, long, float, mpq, mpf)
complextypes = (complex, mpqc, mpc)

inttypes_set = frozenset(inttypes)
rationaltypes_set = frozenset(rationaltypes)
realtypes_set = frozenset(realtypes)
complextypes_set = frozenset(complextypes)
numbertypes_set = frozenset(numbertypes)

def number_div(Algebra, a, b):
    if type(b) in inttypes_set:
        if not b:
            if a:
                return Infinity(Infinity(0))
            return Infinity(0)
        if b == 1:
            return a
        if type(a) in inttypes_set:
            return normalized_fraction(a, b)
    return a / b

def div(a, b):
    """Safely compute a/b.

    If a or b is an integer, this function makes sure to convert it to
    a rational.
    """
    if type(b) in inttypes_set:
        if not b:
            return Infinity(a)
            raise ZeroDivisionError('%r / %r' % (a, b))
        if b == 1:
            return a
        if type(a) in inttypes_set:
            return normalized_fraction(a, b)
    return a / b

def int_root(y, n):
    """ Return a pair ``(floor(y**(1/n)), x**n == y)``.

    Given integers y and n, return a tuple containing ``x =
    floor(y**(1/n))`` and a boolean indicating whether ``x**n == y``
    exactly.
    """
    if y < 0: raise ValueError, "y must not be negative"
    if n < 1: raise ValueError, "n must be positive"
    if y in (0, 1): return y, True
    if n == 1: return y, True
    if n > y: return 1, False
    # Get initial estimate for Newton's method. Care must be taken to
    # avoid overflow
    try:
        guess = int(y ** (1./n)+0.5)
    except OverflowError:
        try:
            guess = int(math.exp(math.log(y)/n)+0.5)
        except OverflowError:
            guess = 1 << int(math.log(y, 2)/n)
    # Newton iteration
    xprev, x = -1, guess
    while 1:
        t = x**(n-1)
        xprev, x = x, x - (t*x-y)//(n*t)
        if abs(x - xprev) < 1:
            break
    # Compensate
    t = x**n
    while t > y:
        x -= 1
        t = x**n
    return x, t == y

def try_power(x, y):
    """\
    Attempt to compute ``x**y`` where ``x`` and ``y`` must be of the
    types int, long, mpq, mpf, or mpqc. The function
    returns::

      z, symbolic

    where ``z`` is a number (i.e. a complex rational) and ``symbolic``
    is a list of ``(b, e)`` pairs representing symbolic factors
    ``b**e``.

    Examples::
    
      try_power(3, 2) --> (9, [])
      try_power(2, 1/2) --> (1, [(2, 1/2)])
      try_power(45, 1/2) --> (3, [(5, 1/2)])
    """
    if not y or x == 1:
        # (anything)**0 -> 1
        # 1**(anything) -> 1
        return 1, []
    if isinstance(x, Infinity) or isinstance(y, Infinity):
        return x**y, []
    if isinstance(y, inttypes):
        if y >= 0:
            return x**y, []
        elif not x:
            return Infinity.get_zoo(), []
        elif isinstance(x, inttypes):
            return mpq((1, x**(-y))), []
        return x**y, []
    elif isinstance(x, inttypes) and isinstance(y, mpq):
        if x < 0:
            if x==-1:
                p, q = y
                if q==2:
                    return mpqc(0, 1)**p, []
                return 1, [(x,y)]
            else:
                z, sym = try_power(-x, y)
                z1, sym1 = try_power(-1, y)
                return z * z1, sym+sym1
        else:
            p, q = y
            r, exact = int_root(x, q)
            if exact:
                if r==1 or not p:
                    return 1, []
                g = r**p if p>0 else mpq((1, r**(-p)))
                return g, []
    elif isinstance(x, mpq) and isinstance(y, mpq):
        a, b = x
        r, rsym = try_power(a, y)
        s, ssym = try_power(b, y)
        ssym = [(b, -e) for b, e in ssym]
        return (div(r,s), rsym + ssym)
    elif isinstance(x, mpf) or isinstance(y, mpf):
        return x ** y, []
    return 1, [(x, y)]

from .number_theory import gcd
from .evalf import evalf
from .infinity import Infinity

from .methods import (\
    fraction_add, fraction_sub, fraction_rsub, fraction_mul,
    fraction_div, fraction_rdiv, fraction_pow,
    fraction_lt, fraction_le, fraction_gt, fraction_ge,
    complex_add, complex_sub, complex_rsub, complex_mul,
    complex_div, complex_rdiv, complex_pow)

mpq.__add__ = mpq.__radd__ = fraction_add
mpq.__sub__ = fraction_sub
mpq.__rsub__ = fraction_rsub
mpq.__mul__ = mpq.__rmul__ = fraction_mul
mpq.__div__ = fraction_div
mpq.__rdiv__ = fraction_rdiv
mpq.__pow__ = fraction_pow
mpq.__lt__ = fraction_lt
mpq.__le__ = fraction_le
mpq.__gt__ = fraction_gt
mpq.__ge__ = fraction_ge

mpqc.__add__ = mpqc.__radd__ = complex_add
mpqc.__sub__ = complex_sub
mpqc.__rsub__ = complex_rsub
mpqc.__mul__ = mpqc.__rmul__ = complex_mul
mpqc.__div__ = complex_div
mpqc.__rdiv__ = complex_rdiv
mpqc.__pow__ = complex_pow



import pymaclab as pm
import pymaclab.modfiles.models as models

rbc = pm.newMOD(models.stable.rbc1_num,mesg=False,ncpus='auto')

def test_all():
    # Do for paramdic
    eta_key = 'eta'
    eta_old = 2.0
    eta_new = 5.0
    rho_key = 'rho'
    rho_old = 0.36
    rho_new = 0.35
    rbc.updaters_queued.paramdic[eta_key] = eta_new
    rbc.updaters_queued.paramdic[rho_key] = rho_new
    
    # Do for nlsubsdic
    U_key = '@U(t)'
    U_old = 'c(t)**(1-eta)/(1-eta)'
    U_new = 'c(t)**(1-eta*1.01)/(1-eta*1.01)'
    F_key = '@F(t)'
    F_old = 'z(t)*k(t-1)**rho'
    F_new = 'z(t)*k(t-1)**rho*1.01'
    rbc.updaters_queued.nlsubsdic[U_key] = U_new
    rbc.updaters_queued.nlsubsdic[F_key] = F_new
    
    # Do for vardic
    var_key = ['c(t)','consumption']
    indexo = rbc.vardic['con']['var'].index(var_key)
    var_old = 'bk'
    var_new = 'cf'
    rbc.updaters_queued.vardic['con']['mod'][indexo][1] = var_new
    
    
    # NOW process the queue
    rbc.updaters_queued.process_queue()
    
    
    # Did it work?
    # For paramdic
    assert rbc.updaters_queued.paramdic.wrapobj[eta_key] == eta_new
    assert rbc.updaters_queued.paramdic[eta_key] == eta_new
    assert rbc.paramdic[eta_key] == eta_new
    
    assert rbc.updaters_queued.paramdic.wrapobj[rho_key] == rho_new
    assert rbc.updaters_queued.paramdic[rho_key] == rho_new
    assert rbc.paramdic[rho_key] == rho_new
    
    # For nlsubsdic
    assert rbc.updaters_queued.nlsubsdic.wrapobj[U_key] == U_new
    assert rbc.updaters_queued.nlsubsdic[U_key] == U_new
    assert rbc.nlsubsdic[U_key] == U_new
    
    assert rbc.updaters_queued.nlsubsdic.wrapobj[F_key] == F_new
    assert rbc.updaters_queued.nlsubsdic[F_key] == F_new
    assert rbc.nlsubsdic[F_key] == F_new
    
    # For vardic
    assert rbc.updaters_queued.vardic.wrapobj['con']['mod'][indexo][1] == var_new
    assert rbc.updaters_queued.vardic['con']['mod'][indexo][1] == var_new
    assert rbc.vardic['con']['mod'][indexo][1] == var_new
    
    



from __future__ import with_statement

__all__ = ['EQ', 'NE', 'LT', 'LE', 'GT', 'GE']

from ..core import init_module, classes, SymbolicEquality
init_module.import_heads()
init_module.import_numbers()

from .base import Head, BinaryHead, heads_precedence, Expr

class RelationalHead(BinaryHead):

    def walk(self, func, cls, data, target):
        lhs, rhs = data
        lhs1 = lhs.head.walk(func, classes.Calculus, lhs.data, lhs)
        rhs1 = rhs.head.walk(func, classes.Calculus, rhs.data, rhs)
        if lhs1 is lhs and rhs1 is rhs:
            return func(cls, data, target)
        r = self.new(cls, (lhs1, rhs1))
        return func(cls, r.head, r.data, r)

class EqHead(RelationalHead):

    op_symbol = '=='
    op_mth = '__eq__'

    def new(self, cls, (lhs, rhs)):
        if lhs.head is NUMBER and rhs.head is NUMBER:
            return cls(lhs.data == rhs.data)
        return cls(self, (lhs, rhs))

    def reevaluate(self, cls, (lhs, rhs)):
        with SymbolicEquality(type(lhs), type(rhs)):
            return lhs == rhs

    def __repr__(self): return 'EQ'

    def nonzero(self, cls, (lhs, rhs)):
        if isinstance(lhs, Expr):
            lhs = lhs.as_lowlevel()
        if isinstance(rhs, Expr):
            rhs = rhs.as_lowlevel()
        if isinstance(lhs, numbertypes) and isinstance(rhs, numbertypes):
            return lhs == rhs
        return True

class NeHead(RelationalHead):

    op_symbol = '!='
    op_mth = '__ne__'

    def new(self, cls, (lhs, rhs)):
        if lhs.head is NUMBER and rhs.head is NUMBER:
            return cls(lhs.data != rhs.data)
        return cls(self, (lhs, rhs))

    def reevaluate(self, cls, (lhs, rhs)):
        with SymbolicEquality(type(lhs), type(rhs)):
            return lhs != rhs

    def __repr__(self): return 'NE'

    def nonzero(self, cls, (lhs, rhs)):
        if isinstance(lhs, Expr):
            lhs = lhs.as_lowlevel()
        if isinstance(rhs, Expr):
            rhs = rhs.as_lowlevel()
        if isinstance(lhs, numbertypes) and isinstance(rhs, numbertypes):
            return lhs != rhs
        return True
    
class LtHead(RelationalHead):

    op_symbol = '<'
    op_mth = '__lt__'
    def __repr__(self): return 'LT'

    def new(self, cls, (lhs, rhs), evaluate=True):
        if lhs.head is NUMBER and rhs.head is NUMBER:
            return cls(lhs.data < rhs.data)
        return cls(self, (lhs, rhs))

    def reevaluate(self, cls, (lhs, rhs)):
        return lhs < rhs

    def nonzero(self, cls, (lhs, rhs)):
        if isinstance(lhs, Expr):
            lhs = lhs.as_lowlevel()
        if isinstance(rhs, Expr):
            rhs = rhs.as_lowlevel()
        if isinstance(lhs, numbertypes) and isinstance(rhs, numbertypes):
            return lhs < rhs
        return True

class LeHead(RelationalHead):

    op_symbol = '<='
    op_mth = '__le__'

    def new(self, cls, (lhs, rhs)):
        if lhs.head is NUMBER and rhs.head is NUMBER:
            return cls(lhs.data <= rhs.data)
        return cls(self, (lhs, rhs))

    def reevaluate(self, cls, (lhs, rhs)):
        return lhs <= rhs
    
    def __repr__(self): return 'LE'

    def nonzero(self, cls, (lhs, rhs)):
        if isinstance(lhs, Expr):
            lhs = lhs.as_lowlevel()
        if isinstance(rhs, Expr):
            rhs = rhs.as_lowlevel()
        if isinstance(lhs, numbertypes) and isinstance(rhs, numbertypes):
            return lhs <= rhs
        return True

class GtHead(RelationalHead):

    op_symbol = '>'
    op_mth = '__gt__'

    def new(self, cls, (lhs, rhs)):
        if lhs.head is NUMBER and rhs.head is NUMBER:
            return cls(lhs.data > rhs.data)
        return cls(self, (lhs, rhs))


    def reevaluate(self, cls, (lhs, rhs)):
        return lhs > rhs
    
    def __repr__(self): return 'GT'

    def nonzero(self, cls, (lhs, rhs)):
        if isinstance(lhs, Expr):
            lhs = lhs.as_lowlevel()
        if isinstance(rhs, Expr):
            rhs = rhs.as_lowlevel()
        if isinstance(lhs, numbertypes) and isinstance(rhs, numbertypes):
            return lhs > rhs
        return True
    
class GeHead(RelationalHead):

    op_symbol = '>='
    op_mth = '__ge__'

    def new(self, cls, (lhs, rhs)):
        if lhs.head is NUMBER and rhs.head is NUMBER:
            return cls(lhs.data >= rhs.data)
        return cls(self, (lhs, rhs))

    def reevaluate(self, cls, (lhs, rhs)):
        return lhs >= rhs
    
    def __repr__(self): return 'GE'

    def nonzero(self, cls, (lhs, rhs)):
        if isinstance(lhs, Expr):
            lhs = lhs.as_lowlevel()
        if isinstance(rhs, Expr):
            rhs = rhs.as_lowlevel()
        if isinstance(lhs, numbertypes) and isinstance(rhs, numbertypes):
            return lhs >= rhs
        return True

EQ = EqHead()
NE = NeHead()
LT = LtHead()
LE = LeHead()
GT = GtHead()
GE = GeHead()




from sympycore import Ring, heads, core

P = Ring.Polynom

def test_zero():
    x,y,z = map(Ring, 'xyz')
    zero =  P(variables=(x,y))
    assert zero.pair==(heads.EXP_COEFF_DICT, ((x,y), {})), `zero.pair`
    assert zero==0
    zero = ((zero + x) - x)
    assert zero.pair==(heads.EXP_COEFF_DICT, ((x,y), {})), `zero.pair`
    assert zero==0,`zero`

    
def test_one():
    x,y,z = map(Ring, 'xyz')
    one = Ring.Polynom(1,variables=(x,y))
    assert one.pair == (heads.EXP_COEFF_DICT, ((x, y), {core.IntegerList([0,0]):1})), `one.pair`
    assert one==1
    one = Ring.Polynom(x,variables=(x,y))/x
    assert one.pair == (heads.EXP_COEFF_DICT, ((), {core.IntegerList([]):1})), `one.pair`
    assert one==1

def test_simple():
    x,y,z = map(Ring, 'xyz')
    



from mpmath import *

def test_interval_identity():
    iv.dps = 15
    assert mpi(2) == mpi(2, 2)
    assert mpi(2) != mpi(-2, 2)
    assert not (mpi(2) != mpi(2, 2))
    assert mpi(-1, 1) == mpi(-1, 1)
    assert str(mpi('0.1')) == "[0.099999999999999991673, 0.10000000000000000555]"
    assert repr(mpi('0.1')) == "mpi('0.099999999999999992', '0.10000000000000001')"
    u = mpi(-1, 3)
    assert -1 in u
    assert 2 in u
    assert 3 in u
    assert -1.1 not in u
    assert 3.1 not in u
    assert mpi(-1, 3) in u
    assert mpi(0, 1) in u
    assert mpi(-1.1, 2) not in u
    assert mpi(2.5, 3.1) not in u
    w = mpi(-inf, inf)
    assert mpi(-5, 5) in w
    assert mpi(2, inf) in w
    assert mpi(0, 2) in mpi(0, 10)
    assert not (3 in mpi(-inf, 0))

def test_interval_arithmetic():
    iv.dps = 15
    assert mpi(2) + mpi(3,4) == mpi(5,6)
    assert mpi(1, 2)**2 == mpi(1, 4)
    assert mpi(1) + mpi(0, 1e-50) == mpi(1, mpf('1.0000000000000002'))
    x = 1 / (1 / mpi(3))
    assert x.a < 3 < x.b
    x = mpi(2) ** mpi(0.5)
    iv.dps += 5
    sq = iv.sqrt(2)
    iv.dps -= 5
    assert x.a < sq < x.b
    assert mpi(1) / mpi(1, inf)
    assert mpi(2, 3) / inf == mpi(0, 0)
    assert mpi(0) / inf == 0
    assert mpi(0) / 0 == mpi(-inf, inf)
    assert mpi(inf) / 0 == mpi(-inf, inf)
    assert mpi(0) * inf == mpi(-inf, inf)
    assert 1 / mpi(2, inf) == mpi(0, 0.5)
    assert str((mpi(50, 50) * mpi(-10, -10)) / 3) == \
        '[-166.66666666666668561, -166.66666666666665719]'
    assert mpi(0, 4) ** 3 == mpi(0, 64)
    assert mpi(2,4).mid == 3
    iv.dps = 30
    a = mpi(iv.pi)
    iv.dps = 15
    b = +a
    assert b.a < a.a
    assert b.b > a.b
    a = mpi(iv.pi)
    assert a == +a
    assert abs(mpi(-1,2)) == mpi(0,2)
    assert abs(mpi(0.5,2)) == mpi(0.5,2)
    assert abs(mpi(-3,2)) == mpi(0,3)
    assert abs(mpi(-3,-0.5)) == mpi(0.5,3)
    assert mpi(0) * mpi(2,3) == mpi(0)
    assert mpi(2,3) * mpi(0) == mpi(0)
    assert mpi(1,3).delta == 2
    assert mpi(1,2) - mpi(3,4) == mpi(-3,-1)
    assert mpi(-inf,0) - mpi(0,inf) == mpi(-inf,0)
    assert mpi(-inf,0) - mpi(-inf,inf) == mpi(-inf,inf)
    assert mpi(0,inf) - mpi(-inf,1) == mpi(-1,inf)

def test_interval_mul():
    assert mpi(-1, 0) * inf == mpi(-inf, 0)
    assert mpi(-1, 0) * -inf == mpi(0, inf)
    assert mpi(0, 1) * inf == mpi(0, inf)
    assert mpi(0, 1) * mpi(0, inf) == mpi(0, inf)
    assert mpi(-1, 1) * inf == mpi(-inf, inf)
    assert mpi(-1, 1) * mpi(0, inf) == mpi(-inf, inf)
    assert mpi(-1, 1) * mpi(-inf, inf) == mpi(-inf, inf)
    assert mpi(-inf, 0) * mpi(0, 1) == mpi(-inf, 0)
    assert mpi(-inf, 0) * mpi(0, 0) * mpi(-inf, 0)
    assert mpi(-inf, 0) * mpi(-inf, inf) == mpi(-inf, inf)
    assert mpi(-5,0)*mpi(-32,28) == mpi(-140,160)
    assert mpi(2,3) * mpi(-1,2) == mpi(-3,6)
    # Should be undefined?
    assert mpi(inf, inf) * 0 == mpi(-inf, inf)
    assert mpi(-inf, -inf) * 0 == mpi(-inf, inf)
    assert mpi(0) * mpi(-inf,2) == mpi(-inf,inf)
    assert mpi(0) * mpi(-2,inf) == mpi(-inf,inf)
    assert mpi(-2,inf) * mpi(0) == mpi(-inf,inf)
    assert mpi(-inf,2) * mpi(0) == mpi(-inf,inf)

def test_interval_pow():
    assert mpi(3)**2 == mpi(9, 9)
    assert mpi(-3)**2 == mpi(9, 9)
    assert mpi(-3, 1)**2 == mpi(0, 9)
    assert mpi(-3, -1)**2 == mpi(1, 9)
    assert mpi(-3, -1)**3 == mpi(-27, -1)
    assert mpi(-3, 1)**3 == mpi(-27, 1)
    assert mpi(-2, 3)**2 == mpi(0, 9)
    assert mpi(-3, 2)**2 == mpi(0, 9)
    assert mpi(4) ** -1 == mpi(0.25, 0.25)
    assert mpi(-4) ** -1 == mpi(-0.25, -0.25)
    assert mpi(4) ** -2 == mpi(0.0625, 0.0625)
    assert mpi(-4) ** -2 == mpi(0.0625, 0.0625)
    assert mpi(0, 1) ** inf == mpi(0, 1)
    assert mpi(0, 1) ** -inf == mpi(1, inf)
    assert mpi(0, inf) ** inf == mpi(0, inf)
    assert mpi(0, inf) ** -inf == mpi(0, inf)
    assert mpi(1, inf) ** inf == mpi(1, inf)
    assert mpi(1, inf) ** -inf == mpi(0, 1)
    assert mpi(2, 3) ** 1 == mpi(2, 3)
    assert mpi(2, 3) ** 0 == 1
    assert mpi(1,3) ** mpi(2) == mpi(1,9)

def test_interval_sqrt():
    assert mpi(4) ** 0.5 == mpi(2)

def test_interval_div():
    assert mpi(0.5, 1) / mpi(-1, 0) == mpi(-inf, -0.5)
    assert mpi(0, 1) / mpi(0, 1) == mpi(0, inf)
    assert mpi(inf, inf) / mpi(inf, inf) == mpi(0, inf)
    assert mpi(inf, inf) / mpi(2, inf) == mpi(0, inf)
    assert mpi(inf, inf) / mpi(2, 2) == mpi(inf, inf)
    assert mpi(0, inf) / mpi(2, inf) == mpi(0, inf)
    assert mpi(0, inf) / mpi(2, 2) == mpi(0, inf)
    assert mpi(2, inf) / mpi(2, 2) == mpi(1, inf)
    assert mpi(2, inf) / mpi(2, inf) == mpi(0, inf)
    assert mpi(-4, 8) / mpi(1, inf) == mpi(-4, 8)
    assert mpi(-4, 8) / mpi(0.5, inf) == mpi(-8, 16)
    assert mpi(-inf, 8) / mpi(0.5, inf) == mpi(-inf, 16)
    assert mpi(-inf, inf) / mpi(0.5, inf) == mpi(-inf, inf)
    assert mpi(8, inf) / mpi(0.5, inf) == mpi(0, inf)
    assert mpi(-8, inf) / mpi(0.5, inf) == mpi(-16, inf)
    assert mpi(-4, 8) / mpi(inf, inf) == mpi(0, 0)
    assert mpi(0, 8) / mpi(inf, inf) == mpi(0, 0)
    assert mpi(0, 0) / mpi(inf, inf) == mpi(0, 0)
    assert mpi(-inf, 0) / mpi(inf, inf) == mpi(-inf, 0)
    assert mpi(-inf, 8) / mpi(inf, inf) == mpi(-inf, 0)
    assert mpi(-inf, inf) / mpi(inf, inf) == mpi(-inf, inf)
    assert mpi(-8, inf) / mpi(inf, inf) == mpi(0, inf)
    assert mpi(0, inf) / mpi(inf, inf) == mpi(0, inf)
    assert mpi(8, inf) / mpi(inf, inf) == mpi(0, inf)
    assert mpi(inf, inf) / mpi(inf, inf) == mpi(0, inf)
    assert mpi(-1, 2) / mpi(0, 1) == mpi(-inf, +inf)
    assert mpi(0, 1) / mpi(0, 1) == mpi(0.0, +inf)
    assert mpi(-1, 0) / mpi(0, 1) == mpi(-inf, 0.0)
    assert mpi(-0.5, -0.25) / mpi(0, 1) == mpi(-inf, -0.25)
    assert mpi(0.5, 1) / mpi(0, 1) == mpi(0.5, +inf)
    assert mpi(0.5, 4) / mpi(0, 1) == mpi(0.5, +inf)
    assert mpi(-1, -0.5) / mpi(0, 1) == mpi(-inf, -0.5)
    assert mpi(-4, -0.5) / mpi(0, 1) == mpi(-inf, -0.5)
    assert mpi(-1, 2) / mpi(-2, 0.5) == mpi(-inf, +inf)
    assert mpi(0, 1) / mpi(-2, 0.5) == mpi(-inf, +inf)
    assert mpi(-1, 0) / mpi(-2, 0.5) == mpi(-inf, +inf)
    assert mpi(-0.5, -0.25) / mpi(-2, 0.5) == mpi(-inf, +inf)
    assert mpi(0.5, 1) / mpi(-2, 0.5) == mpi(-inf, +inf)
    assert mpi(0.5, 4) / mpi(-2, 0.5) == mpi(-inf, +inf)
    assert mpi(-1, -0.5) / mpi(-2, 0.5) == mpi(-inf, +inf)
    assert mpi(-4, -0.5) / mpi(-2, 0.5) == mpi(-inf, +inf)
    assert mpi(-1, 2) / mpi(-1, 0) == mpi(-inf, +inf)
    assert mpi(0, 1) / mpi(-1, 0) == mpi(-inf, 0.0)
    assert mpi(-1, 0) / mpi(-1, 0) == mpi(0.0, +inf)
    assert mpi(-0.5, -0.25) / mpi(-1, 0) == mpi(0.25, +inf)
    assert mpi(0.5, 1) / mpi(-1, 0) == mpi(-inf, -0.5)
    assert mpi(0.5, 4) / mpi(-1, 0) == mpi(-inf, -0.5)
    assert mpi(-1, -0.5) / mpi(-1, 0) == mpi(0.5, +inf)
    assert mpi(-4, -0.5) / mpi(-1, 0) == mpi(0.5, +inf)
    assert mpi(-1, 2) / mpi(0.5, 1) == mpi(-2.0, 4.0)
    assert mpi(0, 1) / mpi(0.5, 1) == mpi(0.0, 2.0)
    assert mpi(-1, 0) / mpi(0.5, 1) == mpi(-2.0, 0.0)
    assert mpi(-0.5, -0.25) / mpi(0.5, 1) == mpi(-1.0, -0.25)
    assert mpi(0.5, 1) / mpi(0.5, 1) == mpi(0.5, 2.0)
    assert mpi(0.5, 4) / mpi(0.5, 1) == mpi(0.5, 8.0)
    assert mpi(-1, -0.5) / mpi(0.5, 1) == mpi(-2.0, -0.5)
    assert mpi(-4, -0.5) / mpi(0.5, 1) == mpi(-8.0, -0.5)
    assert mpi(-1, 2) / mpi(-2, -0.5) == mpi(-4.0, 2.0)
    assert mpi(0, 1) / mpi(-2, -0.5) == mpi(-2.0, 0.0)
    assert mpi(-1, 0) / mpi(-2, -0.5) == mpi(0.0, 2.0)
    assert mpi(-0.5, -0.25) / mpi(-2, -0.5) == mpi(0.125, 1.0)
    assert mpi(0.5, 1) / mpi(-2, -0.5) == mpi(-2.0, -0.25)
    assert mpi(0.5, 4) / mpi(-2, -0.5) == mpi(-8.0, -0.25)
    assert mpi(-1, -0.5) / mpi(-2, -0.5) == mpi(0.25, 2.0)
    assert mpi(-4, -0.5) / mpi(-2, -0.5) == mpi(0.25, 8.0)
    # Should be undefined?
    assert mpi(0, 0) / mpi(0, 0) == mpi(-inf, inf)
    assert mpi(0, 0) / mpi(0, 1) == mpi(-inf, inf)

def test_interval_cos_sin():
    iv.dps = 15
    cos = iv.cos
    sin = iv.sin
    tan = iv.tan
    pi = iv.pi
    # Around 0
    assert cos(mpi(0)) == 1
    assert sin(mpi(0)) == 0
    assert cos(mpi(0,1)) == mpi(0.54030230586813965399, 1.0)
    assert sin(mpi(0,1)) == mpi(0, 0.8414709848078966159)
    assert cos(mpi(1,2)) == mpi(-0.4161468365471424069, 0.54030230586813976501)
    assert sin(mpi(1,2)) == mpi(0.84147098480789650488, 1.0)
    assert sin(mpi(1,2.5)) == mpi(0.59847214410395643824, 1.0)
    assert cos(mpi(-1, 1)) == mpi(0.54030230586813965399, 1.0)
    assert cos(mpi(-1, 0.5)) == mpi(0.54030230586813965399, 1.0)
    assert cos(mpi(-1, 1.5)) == mpi(0.070737201667702906405, 1.0)
    assert sin(mpi(-1,1)) == mpi(-0.8414709848078966159, 0.8414709848078966159)
    assert sin(mpi(-1,0.5)) == mpi(-0.8414709848078966159, 0.47942553860420300538)
    assert mpi(-0.8414709848078966159, 1.00000000000000002e-100) in sin(mpi(-1,1e-100))
    assert mpi(-2.00000000000000004e-100, 1.00000000000000002e-100) in sin(mpi(-2e-100,1e-100))
    # Same interval
    assert cos(mpi(2, 2.5))
    assert cos(mpi(3.5, 4)) == mpi(-0.93645668729079634129, -0.65364362086361182946)
    assert cos(mpi(5, 5.5)) == mpi(0.28366218546322624627, 0.70866977429126010168)
    assert mpi(0.59847214410395654927, 0.90929742682568170942) in sin(mpi(2, 2.5))
    assert sin(mpi(3.5, 4)) == mpi(-0.75680249530792831347, -0.35078322768961983646)
    assert sin(mpi(5, 5.5)) == mpi(-0.95892427466313856499, -0.70554032557039181306)
    # Higher roots
    iv.dps = 55
    w = 4*10**50 + mpi(0.5)
    for p in [15, 40, 80]:
        iv.dps = p
        assert 0 in sin(4*mpi(pi))
        assert 0 in sin(4*10**50*mpi(pi))
        assert 0 in cos((4+0.5)*mpi(pi))
        assert 0 in cos(w*mpi(pi))
        assert 1 in cos(4*mpi(pi))
        assert 1 in cos(4*10**50*mpi(pi))
    iv.dps = 15
    assert cos(mpi(2,inf)) == mpi(-1,1)
    assert sin(mpi(2,inf)) == mpi(-1,1)
    assert cos(mpi(-inf,2)) == mpi(-1,1)
    assert sin(mpi(-inf,2)) == mpi(-1,1)
    u = tan(mpi(0.5,1))
    assert mpf(u.a).ae(mp.tan(0.5))
    assert mpf(u.b).ae(mp.tan(1))
    v = iv.cot(mpi(0.5,1))
    assert mpf(v.a).ae(mp.cot(1))
    assert mpf(v.b).ae(mp.cot(0.5))
    # Sanity check of evaluation at n*pi and (n+1/2)*pi
    for n in range(-5,7,2):
        x = iv.cos(n*iv.pi)
        assert -1 in x
        assert x >= -1
        assert x != -1
        x = iv.sin((n+0.5)*iv.pi)
        assert -1 in x
        assert x >= -1
        assert x != -1
    for n in range(-6,8,2):
        x = iv.cos(n*iv.pi)
        assert 1 in x
        assert x <= 1
        if n:
            assert x != 1
        x = iv.sin((n+0.5)*iv.pi)
        assert 1 in x
        assert x <= 1
        assert x != 1
    for n in range(-6,7):
        x = iv.cos((n+0.5)*iv.pi)
        assert x.a < 0 < x.b
        x = iv.sin(n*iv.pi)
        if n:
            assert x.a < 0 < x.b

def test_interval_complex():
    # TODO: many more tests
    iv.dps = 15
    mp.dps = 15
    assert iv.mpc(2,3) == 2+3j
    assert iv.mpc(2,3) != 2+4j
    assert iv.mpc(2,3) != 1+3j
    assert 1+3j in iv.mpc([1,2],[3,4])
    assert 2+5j not in iv.mpc([1,2],[3,4])
    assert iv.mpc(1,2) + 1j == 1+3j
    assert iv.mpc([1,2],[2,3]) + 2+3j == iv.mpc([3,4],[5,6])
    assert iv.mpc([2,4],[4,8]) / 2 == iv.mpc([1,2],[2,4])
    assert iv.mpc([1,2],[2,4]) * 2j == iv.mpc([-8,-4],[2,4])
    assert iv.mpc([2,4],[4,8]) / 2j == iv.mpc([2,4],[-2,-1])
    assert iv.exp(2+3j).ae(mp.exp(2+3j))
    assert iv.log(2+3j).ae(mp.log(2+3j))
    assert (iv.mpc(2,3) ** iv.mpc(0.5,2)).ae(mp.mpc(2,3) ** mp.mpc(0.5,2))
    assert 1j in (iv.mpf(-1) ** 0.5)
    assert 1j in (iv.mpc(-1) ** 0.5)
    assert abs(iv.mpc(0)) == 0
    assert abs(iv.mpc(inf)) == inf
    assert abs(iv.mpc(3,4)) == 5
    assert abs(iv.mpc(4)) == 4
    assert abs(iv.mpc(0,4)) == 4
    assert abs(iv.mpc(0,[2,3])) == iv.mpf([2,3])
    assert abs(iv.mpc(0,[-3,2])) == iv.mpf([0,3])
    assert abs(iv.mpc([3,5],[4,12])) == iv.mpf([5,13])
    assert abs(iv.mpc([3,5],[-4,12])) == iv.mpf([3,13])
    assert iv.mpc(2,3) ** 0 == 1
    assert iv.mpc(2,3) ** 1 == (2+3j)
    assert iv.mpc(2,3) ** 2 == (2+3j)**2
    assert iv.mpc(2,3) ** 3 == (2+3j)**3
    assert iv.mpc(2,3) ** 4 == (2+3j)**4
    assert iv.mpc(2,3) ** 5 == (2+3j)**5
    assert iv.mpc(2,2) ** (-1) == (2+2j) ** (-1)
    assert iv.mpc(2,2) ** (-2) == (2+2j) ** (-2)
    assert iv.cos(2).ae(mp.cos(2))
    assert iv.sin(2).ae(mp.sin(2))
    assert iv.cos(2+3j).ae(mp.cos(2+3j))
    assert iv.sin(2+3j).ae(mp.sin(2+3j))

def test_interval_complex_arg():
    assert iv.arg(3) == 0
    assert iv.arg(0) == 0
    assert iv.arg([0,3]) == 0
    assert iv.arg(-3).ae(pi)
    assert iv.arg(2+3j).ae(iv.arg(2+3j))
    z = iv.mpc([-2,-1],[3,4])
    t = iv.arg(z)
    assert t.a.ae(mp.arg(-1+4j))
    assert t.b.ae(mp.arg(-2+3j))
    z = iv.mpc([-2,1],[3,4])
    t = iv.arg(z)
    assert t.a.ae(mp.arg(1+3j))
    assert t.b.ae(mp.arg(-2+3j))
    z = iv.mpc([1,2],[3,4])
    t = iv.arg(z)
    assert t.a.ae(mp.arg(2+3j))
    assert t.b.ae(mp.arg(1+4j))
    z = iv.mpc([1,2],[-2,3])
    t = iv.arg(z)
    assert t.a.ae(mp.arg(1-2j))
    assert t.b.ae(mp.arg(1+3j))
    z = iv.mpc([1,2],[-4,-3])
    t = iv.arg(z)
    assert t.a.ae(mp.arg(1-4j))
    assert t.b.ae(mp.arg(2-3j))
    z = iv.mpc([-1,2],[-4,-3])
    t = iv.arg(z)
    assert t.a.ae(mp.arg(-1-3j))
    assert t.b.ae(mp.arg(2-3j))
    z = iv.mpc([-2,-1],[-4,-3])
    t = iv.arg(z)
    assert t.a.ae(mp.arg(-2-3j))
    assert t.b.ae(mp.arg(-1-4j))
    z = iv.mpc([-2,-1],[-3,3])
    t = iv.arg(z)
    assert t.a.ae(-mp.pi)
    assert t.b.ae(mp.pi)
    z = iv.mpc([-2,2],[-3,3])
    t = iv.arg(z)
    assert t.a.ae(-mp.pi)
    assert t.b.ae(mp.pi)

def test_interval_ae():
    iv.dps = 15
    x = iv.mpf([1,2])
    assert x.ae(1) is None
    assert x.ae(1.5) is None
    assert x.ae(2) is None
    assert x.ae(2.01) is False
    assert x.ae(0.99) is False
    x = iv.mpf(3.5)
    assert x.ae(3.5) is True
    assert x.ae(3.5+1e-15) is True
    assert x.ae(3.5-1e-15) is True
    assert x.ae(3.501) is False
    assert x.ae(3.499) is False
    assert x.ae(iv.mpf([3.5,3.501])) is None
    assert x.ae(iv.mpf([3.5,4.5+1e-15])) is None

def test_interval_nstr():
    iv.dps = n = 30
    x = mpi(1, 2)
    # FIXME: error_dps should not be necessary
    assert iv.nstr(x, n, mode='plusminus', error_dps=6) == '1.5 +- 0.5'
    assert iv.nstr(x, n, mode='plusminus', use_spaces=False, error_dps=6) == '1.5+-0.5'
    assert iv.nstr(x, n, mode='percent') == '1.5 (33.33%)'
    assert iv.nstr(x, n, mode='brackets', use_spaces=False) == '[1.0,2.0]'
    assert iv.nstr(x, n, mode='brackets' , brackets=('<', '>')) == '<1.0, 2.0>'
    x = mpi('5.2582327113062393041', '5.2582327113062749951')
    assert iv.nstr(x, n, mode='diff') == '5.2582327113062[393041, 749951]'
    assert iv.nstr(iv.cos(mpi(1)), n, mode='diff', use_spaces=False) == '0.54030230586813971740093660744[2955,3053]'
    assert iv.nstr(mpi('1e123', '1e129'), n, mode='diff') == '[1.0e+123, 1.0e+129]'
    exp = iv.exp
    assert iv.nstr(iv.exp(mpi('5000.1')), n, mode='diff') == '3.2797365856787867069110487[0926, 1191]e+2171'

def test_mpi_from_str():
    iv.dps = 15
    assert iv.convert('1.5 +- 0.5') == mpi(mpf('1.0'), mpf('2.0'))
    assert mpi(1, 2) in iv.convert('1.5 (33.33333333333333333333333333333%)')
    assert iv.convert('[1, 2]') == mpi(1, 2)
    assert iv.convert('1[2, 3]') == mpi(12, 13)
    assert iv.convert('1.[23,46]e-8') == mpi('1.23e-8', '1.46e-8')
    assert iv.convert('12[3.4,5.9]e4') == mpi('123.4e+4', '125.9e4')

def test_interval_gamma():
    mp.dps = 15
    iv.dps = 15
    # TODO: need many more tests
    assert iv.rgamma(0) == 0
    assert iv.fac(0) == 1
    assert iv.fac(1) == 1
    assert iv.fac(2) == 2
    assert iv.fac(3) == 6
    assert iv.gamma(0) == [-inf,inf]
    assert iv.gamma(1) == 1
    assert iv.gamma(2) == 1
    assert iv.gamma(3) == 2
    assert -3.5449077018110320546 in iv.gamma(-0.5)
    assert iv.loggamma(1) == 0
    assert iv.loggamma(2) == 0
    assert 0.69314718055994530942 in iv.loggamma(3)
    # Test tight log-gamma endpoints based on monotonicity
    xs = [iv.mpc([2,3],[1,4]),
          iv.mpc([2,3],[-4,-1]),
          iv.mpc([2,3],[-1,4]),
          iv.mpc([2,3],[-4,1]),
          iv.mpc([2,3],[-4,4]),
          iv.mpc([-3,-2],[2,4]),
          iv.mpc([-3,-2],[-4,-2])]
    for x in xs:
        ys = [mp.loggamma(mp.mpc(x.a,x.c)),
              mp.loggamma(mp.mpc(x.b,x.c)),
              mp.loggamma(mp.mpc(x.a,x.d)),
              mp.loggamma(mp.mpc(x.b,x.d))]
        if 0 in x.imag:
            ys += [mp.loggamma(x.a), mp.loggamma(x.b)]
        min_real = min([y.real for y in ys])
        max_real = max([y.real for y in ys])
        min_imag = min([y.imag for y in ys])
        max_imag = max([y.imag for y in ys])
        z = iv.loggamma(x)
        assert z.a.ae(min_real)
        assert z.b.ae(max_real)
        assert z.c.ae(min_imag)
        assert z.d.ae(max_imag)
